//
//  InitialViewController.swift
//  TraccarClient
//
//  Created by George Makhoul on 30/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit
import Lottie
import Firebase
import FirebaseCore
import FirebaseRemoteConfig

enum shiftStatus: String {
    case notStart
    case inProgress
    case end
    case failed
    case weekend
    case none
}

final class InitialViewController: KNViewController {
    // MARK: - Outlets
    @IBOutlet weak var animationView: LottieAnimationView!
    
    // MARK: - Properties
    private let animationFileName: String = "splash"
    var window: UIWindow?
    private var shift: ShiftModel?
    let userDefaults = UserDefaults.standard
    private var remoteConfig = RemoteConfig.remoteConfig()

    // MARK: - LifeCycle
    override func viewDidLoad() {
        super.viewDidLoad()
        let defaults: [String: NSObject] = ["welcome_message": "Hello, world!" as NSObject]
        remoteConfig.setDefaults(defaults)
        
        remoteConfig.fetch(withExpirationDuration: TimeInterval(1)) { (status, error) -> Void in
          if status == .success {
            print("Config fetched!")
            self.remoteConfig.activate(completion: { (changed, error) in
                let leaderButton = self.remoteConfig.configValue(forKey: "leader_appear").boolValue
                showLeaderButton = leaderButton
            })
          } else {
            print("Config not fetched")
            print("Error: \(error?.localizedDescription ?? "No error available.")")
          }
        }
        view.backgroundColor = .white
        animationView.backgroundColor = .white
        animationView.contentMode = .scaleAspectFill
        animationView.loopMode = .repeat(1)
        animationView.translatesAutoresizingMaskIntoConstraints = false
        
        animationView.play {_ in
            self.navigateTo()
        }

    }
    
    override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews()
    }
    
    func checkShiftStatus(startsAt: String, endsAt: String) -> shiftStatus {
        if userDefaults.bool(forKey: Keys.User.autoTracking) == true && shift?.isHourly == false {
            
            let dateFormatter = DateFormatter()
            dateFormatter.dateFormat = "HH:mm"
            guard let shiftStart = dateFormatter.date(from: startsAt),
                  let shiftEnd = dateFormatter.date(from: endsAt) else {
                print("Invalid shift time format.")
                return .weekend
            }
            
            let calendar = Calendar.current
            let currentTime = Date()
            let currentComponents = calendar.dateComponents([.hour, .minute], from: currentTime)
            let startComponents = calendar.dateComponents([.hour, .minute], from: shiftStart)
            let endComponents = calendar.dateComponents([.hour, .minute], from: shiftEnd)
            
            guard let currentHour = currentComponents.hour, let currentMinute = currentComponents.minute,
                  let startHour = startComponents.hour, let startMinute = startComponents.minute,
                  let endHour = endComponents.hour, let endMinute = endComponents.minute else {
                print("Failed to extract hour and minute components.")
                return .failed
            }
            
            if currentHour < startHour || (currentHour == startHour && currentMinute < startMinute) {
                return .notStart
            } else if currentHour > endHour || (currentHour == endHour && currentMinute > endMinute) {
                return .end
            } else {
                return .inProgress
            }
        }
        else {
            return .none
        }
    }
    
    func navigationMethod (by shift: ShiftModel) {
        
        let shiftStart = shift.getStartTimeForToday()
        let shiftEnd = shift.getEndTimeForToday()
        
        print("shiftStart: \(shiftStart)")
        print("shiftEnds: \(shiftEnd)")
        
        let shiftStatus = checkShiftStatus(startsAt: shiftStart, endsAt: shiftEnd)
        
        switch shiftStatus {
        case .failed:
            print("time formate error")
        case .inProgress:
            let alert = AlertViewController()
            alert.shift = shift
            alert.shiftStatus = shiftStatus
            let navigation = KNNavigationController(rootViewController: alert)
            navigation.modalPresentationStyle = .overFullScreen
            
            window?.rootViewController = navigation
            window?.makeKeyAndVisible()
            
        case .notStart, .end, .none, .weekend:
            let connect = ConnectViewController()
            connect.shift = shift
            connect.shiftStatus = shiftStatus
            let navigation = KNNavigationController(rootViewController: connect)
            navigation.modalPresentationStyle = .overFullScreen
            
            window?.rootViewController = navigation
            window?.makeKeyAndVisible()
        }
    }
}

// MARK: - Methods
extension InitialViewController {
    func navigateTo() {
        if !app.isLoggedIn {
            let login = LoginViewController()
            let navigation = KNNavigationController(rootViewController: login)
            navigation.modalPresentationStyle = .overFullScreen
            
            let coordinator = AuthenticationCoordinator(navigationController: navigation)
            coordinator.step = .signin
            coordinator.start()
            
            window?.rootViewController = navigation
            window?.makeKeyAndVisible()

        } else {
            
            api.check { result in
                if result.success {
                    api.my_shift { [weak self] result in
                        guard let strongSelf = self else { return }
                        if result.success {
                            if let shift = result.data as? ShiftModel {
                                strongSelf.shift = shift
                                api.permissions { result in
                                    if result.success {
                                        api.parameters { result in
                                            if result.success {                                                
                                                strongSelf.navigationMethod(by: shift)
                                            } else {
                                                ok(result)
                                            }
                                        }
                                    } else {
                                        ok(result)
                                    }
                                }
                            }
                        } else {
                            ok(result)
                        }
                    }
                } else {
                    if result.statusCode == 401 {
                        alert(confirm: "Ok", message: result.displayError) {
                            // clear userDefaults
                            UserModel.clearUserFromUserDefaults()
                            DeviceModel.clearUserFromUserDefaults()
                            SignInModel.clearUserFromUserDefaults()
                            
                            // navigate to signin
                            let login = LoginViewController()
                            let navigation = KNNavigationController(rootViewController: login)
                            navigation.modalPresentationStyle = .overFullScreen
                            
                            let coordinator = AuthenticationCoordinator(navigationController: navigation)
                            coordinator.step = .signin
                            coordinator.start()
                            
                            self.window?.rootViewController = navigation
                            self.window?.makeKeyAndVisible()
                        }
                    } else {
                        ok(result)
                    }
                }
            }
            
        }
    }
}
