//
//  KNAlertViewController.swift
//  opa
//
//  Created by George Makhoul on 2/23/22.
//

import UIKit

final class KNAlertViewController: KNViewController {
    
    enum KNAlertViewType {
        case none
        
        case success
        case error
        case decision
    }
    
    // MARK: - Outlets
    @IBOutlet private weak var dimView: UIView!
    
    @IBOutlet private weak var alertView: UIView!
    @IBOutlet private weak var alertView_height: NSLayoutConstraint!
    
    @IBOutlet private weak var confirmationButton: UIButton!
    @IBOutlet private weak var destructiveButton: UIButton!
    
    @IBOutlet private weak var buttonsView_height: NSLayoutConstraint!
    
    @IBOutlet private weak var messageTextView: UITextView!
    @IBOutlet private weak var buttonStackView: UIStackView!
    @IBOutlet private weak var buttonView_leading: NSLayoutConstraint!
    @IBOutlet private weak var buttonsView_trailing: NSLayoutConstraint!
    
    // MARK: - Properties

    private let dimViewColor: UIColor = UIColor.main.withAlphaComponent(0.6)
    private var type: KNAlertViewType = .none
    private var content_height: CGFloat = 150
    
    private var message: String = ""
    
    private var confirmButtonTitle: String = "yes"
    private var destructiveButtonTitle: String = "no"
    
    var shouldDismissAndLogin: Bool = false
    
    var confirmClosure: AlertViewClosure?
    var destructiveClosure: AlertViewClosure?
    
    private let BUTTON_HEIGHT: CGFloat = 52
    private let BUTTON_WIDTH: CGFloat = 128
    
    private var buttonsView_Spacing: CGFloat = 26
    
    // MARK: - Init
    init(type: KNAlertViewType, message: String) {
        super.init()
        
        self.type = type
        self.message = message
    }
    
    init(type: KNAlertViewType, message: String, confirm: String, destructive: String) {
        super.init()
        
        self.type = type
        self.message = message
        
        confirmButtonTitle = confirm
        destructiveButtonTitle = destructive
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    // MARK: - Lifecycle
    override func viewDidLoad() {
        super.viewDidLoad()
        
        view.endEditing(true)
        
        setupUI()
        updateUI()
        updateHeight()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        showPopup()
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
    }
}

// MARK: - UI
extension KNAlertViewController {
    private func setupUI() {
        view.backgroundColor = .clear
        dimView.backgroundColor = dimViewColor
        
        // alert
        alertView.addBorder(radius: 16, width: 0, color: .white)
        
        // text
        messageTextView.textAlignment = .center
        messageTextView.textColor = .main
        messageTextView.font = .medium(16)
        messageTextView.text = ""
        messageTextView.isEditable = false
        messageTextView.isSelectable = false
        messageTextView.textContainerInset = .zero
        messageTextView.textContainer.lineFragmentPadding = 0
        
        // buttons stack
        buttonStackView.spacing = 27
        buttonStackView.backgroundColor = .clear
        
        // buttons
        confirmationButton.set(localized: confirmButtonTitle, color: .white, font: .semibold(17))
        confirmationButton.addCorners(20)
        confirmationButton.backgroundColor = .main
        
        confirmationButton.addTarget(self, action: #selector(confirmationButtonTapped), for: .touchUpInside)
        
        destructiveButton.set(localized: destructiveButtonTitle, color: .main, font: .semibold(17))
        destructiveButton.addCorners(20)
        destructiveButton.backgroundColor = .white

        destructiveButton.addTarget(self, action: #selector(destructiveButtonTapped), for: .touchUpInside)
        
        buttonView_leading.set(buttonsView_Spacing)
        buttonsView_trailing.set(buttonsView_Spacing)

        destructiveButton.hide()
        
        confirmationButton.hide()
        
        buttonsView_height.set(0)
    }
    
    private func updateHeight() {
        var height: CGFloat = 0
        
        height += 28
        height += 20
        height += 28
        height += buttonsView_height.constant
        //@TODO @LOW dynamic height? using a tableView?
        let message_height = messageTextView.height()
        
        if message_height > 500 {
            height += 500
            messageTextView.isScrollEnabled = true
        } else {
            height += message_height
            messageTextView.isScrollEnabled = false
        }
        
        if height < 130 {
            content_height = 130
        } else {
            content_height = height + 20
        }
    }
    
    private func updateUI() {
        messageTextView.text = message
        
        destructiveButton.hide()
        
        confirmationButton.hide()
        
        buttonsView_height.set(0)
        
        switch type {
        case .success, .error:
            let alert_width = alertView.frame.width
            
            buttonsView_Spacing = (alert_width - BUTTON_WIDTH) / 2
            
            buttonView_leading.set(buttonsView_Spacing)
            buttonsView_trailing.set(buttonsView_Spacing)

            confirmationButton.show()
            confirmationButton.set(title: "Ok")
            buttonsView_height.set(BUTTON_HEIGHT)
            
        case .decision:
            buttonsView_Spacing = 26
            
            buttonView_leading.set(buttonsView_Spacing)
            buttonsView_trailing.set(buttonsView_Spacing)

            destructiveButton.show()
            
            confirmationButton.show()
            
            confirmationButton.set(title: "Yes")

            buttonsView_height.set(BUTTON_HEIGHT)
            
            if !destructiveButtonTitle.isEmpty && !confirmButtonTitle.isEmpty {
                confirmationButton.set(localized: confirmButtonTitle, color: .white, font: .semibold(17))
                confirmationButton.addCorners(20)

                confirmationButton.backgroundColor = .main
                
                destructiveButton.set(title: destructiveButtonTitle, color: .main, font: .semibold(17))
                destructiveButton.addCorners(20)
                destructiveButton.backgroundColor = .white
            }
        default:
            break
        }
    }
    
    private func showPopup() {
        var transform = CGAffineTransform.identity
        transform = transform.translatedBy(x: 0, y: UIScreen.main.bounds.height)
        transform = transform.scaledBy(x: 0, y: 0)
        alertView.transform = transform
        
        dimView.alpha = 0
        UIView.animate(withDuration: 0.3) { [weak self] in
            self?.alertView_height.constant = (self?.content_height ?? 0)
            self?.alertView.transform = .identity
            self?.dimView.alpha = 1.0
            
            self?.view.layoutIfNeeded()
        }
    }
    
    @objc private func dismissPopup() {
        UIView.animate(withDuration: 0.3) { [weak self] in
            var transform = CGAffineTransform.identity
            transform = transform.translatedBy(x: 0, y: UIScreen.main.bounds.height)
            transform = transform.scaledBy(x: 0, y: 0)
            self?.alertView.transform = transform
            
            self?.dimView.alpha = 0
            self?.view.layoutIfNeeded()
        } completion: { [weak self] finished in
            if finished {
                self?.dismiss(animated: false) {
                    //
                }
            }
        }
    }
}

// MARK: - Delegate
extension KNAlertViewController {
    @objc func destructiveButtonTapped() {
        if let closure = destructiveClosure {
            closure()
        } else {
            dismissPopup()
        }
    }
    
    @objc func confirmationButtonTapped() {
        if let closure = confirmClosure {
            closure()
        } else {
            dismissPopup()
        }
    }
    
    func hideEverything() {
        view.hide()
    }
}

