//
//  LeaderViewController.swift
//  TraccarClient
//
//  Created by George Makhoul on 24/01/2024.
//  Copyright © 2024 Traccar. All rights reserved.
//

import UIKit
import WebKit

class LeaderViewController: KNViewController, WKUIDelegate {
    
    @IBOutlet weak var leaderNavigationBar: LeaderNavigationView!
    @IBOutlet weak var NavBarHeight: NSLayoutConstraint!
    
    static let eventLogin = Notification.Name("eventLogin")
    static let eventToken = Notification.Name("eventToken")
    static let eventEvent = Notification.Name("eventEvent")
    static let keyToken = "keyToken"
    static let keyEventId = "keyEventId"
    
    var window: UIWindow?
    var webView: WKWebView!
    var initialized = false
    var pendingEventId: String? = nil
    
    @IBOutlet weak var webViewOut: UIView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.isBackButtonHidden = true
        hasCustomNavigation = true
        leaderNavigationBar.delegate = self
        onStart()
    }
    
    private func onStart() {
        
        var urlString = leaderServerURL
        if !urlString.hasSuffix("/") {
            urlString += "/"
        }
        urlString += "api/server"
        
        if let url = URL(string: urlString) {
            let task = URLSession.shared.dataTask(with: url) { data, response, error in
                if error == nil {
                    do {
                        try JSONSerialization.jsonObject(with: data!, options: [])
                        DispatchQueue.main.async {
                            self.onSuccess()
                        }
                    } catch _ as NSError {
                        DispatchQueue.main.async {
                            self.onError()
                        }
                    }
                } else {
                    DispatchQueue.main.async {
                        self.onError()
                    }
                }
            }
            task.resume()
        } else {
            self.onError()
        }
    }
    
    private func onSuccess() {
        UserDefaults.standard.set(leaderServerURL, forKey: "url")
        let userDefaults = UserDefaults.standard

        let statusFrame = UIApplication.shared.statusBarFrame
        var viewFrame = webViewOut.frame
        viewFrame.origin.y = statusFrame.size.height
        viewFrame.size.height -= statusFrame.size.height

        let userContentController = WKUserContentController()
        userContentController.add(self, name: "appInterface")

        let webConfiguration = WKWebViewConfiguration()
        webConfiguration.userContentController = userContentController

        var processPool: WKProcessPool
        if let encodedPool = userDefaults.value(forKey: "pool") as? Data,
           let decodedPool = try? NSKeyedUnarchiver.unarchivedObject(ofClass: WKProcessPool.self, from: encodedPool) {
            processPool = decodedPool
        } else {
            processPool = WKProcessPool()
            let encodedPool = try? NSKeyedArchiver.archivedData(withRootObject: processPool, requiringSecureCoding: true)
            userDefaults.set(encodedPool, forKey: "pool")
        }
        webConfiguration.processPool = processPool

        let group = DispatchGroup()
        if let encodedCookies = userDefaults.value(forKey: "cookies") as? Data,
           let cookies = try? NSKeyedUnarchiver.unarchivedObject(ofClasses: [NSArray.self, HTTPCookie.self], from: encodedCookies) as? [HTTPCookie] {
            if #available(iOS 11.0, *) {
                cookies.forEach { cookie in
                    group.enter()
                    webConfiguration.websiteDataStore.httpCookieStore.setCookie(cookie) {
                        group.leave()
                    }
                }
            }
        }
        
        self.webView = WKWebView(frame: viewFrame, configuration: webConfiguration)
        self.webView.uiDelegate = self
        self.webView.autoresizingMask = [.flexibleWidth, .flexibleHeight]
        
        webViewOut.addSubview(self.webView)
        
        group.notify(queue: DispatchQueue.main) {
            self.initialized = true
            self.loadPage()
        }
        
        NotificationCenter.default.addObserver(self, selector: #selector(onTerminate(_:)), name: UIApplication.willResignActiveNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(onReceive(_:)), name: LeaderViewController.eventToken, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(onEvent(_:)), name: LeaderViewController.eventEvent, object: nil)
    }
    
    
    private func onError() {
        let alert = UIAlertController(title: "Error", message: "Server connection failed", preferredStyle: UIAlertController.Style.alert)
        alert.addAction(UIAlertAction(title: "OK", style: UIAlertAction.Style.default))
        present(alert, animated: true)
    }
    
    private func loadPage() {
        if let urlString = UserDefaults.standard.object(forKey: "url") as? String {
            var urlComponents = URLComponents(string: urlString)
            if let eventId = pendingEventId {
                urlComponents?.queryItems = [URLQueryItem(name: "eventId", value: eventId)]
                pendingEventId = nil
            }
            if let url = urlComponents?.url {
                print("URL***: \(url)")
                self.webView.load(URLRequest(url: url))
            }
        }
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        NotificationCenter.default.removeObserver(self, name: LeaderViewController.eventEvent, object: nil)
        NotificationCenter.default.removeObserver(self, name: LeaderViewController.eventToken, object: nil)
        NotificationCenter.default.removeObserver(self, name: UIApplication.willResignActiveNotification, object: nil)
    }

    @objc func onTerminate(_ notification: Notification) {
        if #available(iOS 11.0, *) {
            self.webView.configuration.websiteDataStore.httpCookieStore.getAllCookies { cookies in
                let encodedCookies = try? NSKeyedArchiver.archivedData(withRootObject: cookies, requiringSecureCoding: true)
                UserDefaults.standard.set(encodedCookies, forKey: "cookies")
            }
        }
    }

    @objc func onReceive(_ notification: Notification) {
        if let token = notification.userInfo?[LeaderViewController.keyToken] {
            let code = "updateNotificationToken && updateNotificationToken('\(token)')"
            webView.evaluateJavaScript(code, completionHandler: nil)
        }
    }

    @objc func onEvent(_ notification: Notification) {
        if let eventId = notification.userInfo?[LeaderViewController.keyEventId] as? String {
            pendingEventId = eventId
            if initialized {
                loadPage()
            }
        }
    }

}

extension LeaderViewController : WKScriptMessageHandler {
    
    func userContentController(_ userContentController: WKUserContentController, didReceive message: WKScriptMessage) {
        if let body = message.body as? String {
            if body.starts(with: "login") {
                leaderNavigationBar.isHidden = true
                NavBarHeight.constant = 0
                if body.count > 6 {
                    let token = String(body[body.index(body.startIndex, offsetBy: 6)...])
                    print("token: \(token)")
                    SecurityManager.shared.saveToken(token)
                }
                NotificationCenter.default.post(name: LeaderViewController.eventLogin, object: nil)
            } else if body.starts(with: "authentication") {
                if let token = SecurityManager.shared.readToken() {
                    let code = "handleLoginToken && handleLoginToken('\(token)')"
                    webView.evaluateJavaScript(code, completionHandler: nil)
                }
            } else if body.starts(with: "logout") {
                SecurityManager.shared.deleteToken()
                self.navigationController?.popViewController(animated: true)
            } else if body.starts(with: "server") {
                
                let urlString = String(body[body.index(body.startIndex, offsetBy: 7)...])
                UserDefaults.standard.set(urlString, forKey: "url")
                if let url = URL(string: urlString) {
                    print("***URL***: \(url)")
                    self.webView.load(URLRequest(url: url))
                }
            }
        }
    }
    
}

// MARK: - LeaderNavigationViewDelegate
extension LeaderViewController: LeaderNavigationViewDelegate {
    func back() {
        self.navigationController?.popViewController(animated: true)
    }
}
