//
//  LoginModelController.swift
//  TraccarClient
//
//  Created by George Makhoul on 26/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import Foundation
import UIKit
import Alamofire
import CoreData

final class LoginModelController {
    // MARK: - Properties
    var fields: [KNField] = []
    var viewController: KNViewController
    private weak var delegate: LoginViewControllerDelegate?
    weak var window: UIWindow?
    weak var coordinator: AuthenticationCoordinator?
    weak var inputProtocol: LoginInputProtocol? {
        didSet {
            setupData()
        }
    }
    private var shift: ShiftModel?
    let userDefaults = UserDefaults.standard

    // MARK: - Init
    init(delegate: LoginViewControllerDelegate?, viewController: KNViewController) {
        self.delegate = delegate
        self.viewController = viewController
    }
    
    private func setupData() {
        let dataProvider = LoginDataProvider()
        fields = dataProvider.signinData()
        
        inputProtocol?.setFields(fields: fields)
    }
    
    func checkShiftStatus(startsAt: String, endsAt: String) -> shiftStatus {
        if userDefaults.bool(forKey: Keys.User.autoTracking) == true && shift?.isHourly == false {
            
            let dateFormatter = DateFormatter()
            dateFormatter.dateFormat = "HH:mm"
            guard let shiftStart = dateFormatter.date(from: startsAt),
                  let shiftEnd = dateFormatter.date(from: endsAt) else {
                print("Invalid shift time format.")
                return .weekend
            }
            
            let calendar = Calendar.current
            let currentTime = Date()
            let currentComponents = calendar.dateComponents([.hour, .minute], from: currentTime)
            let startComponents = calendar.dateComponents([.hour, .minute], from: shiftStart)
            let endComponents = calendar.dateComponents([.hour, .minute], from: shiftEnd)
            
            guard let currentHour = currentComponents.hour, let currentMinute = currentComponents.minute,
                  let startHour = startComponents.hour, let startMinute = startComponents.minute,
                  let endHour = endComponents.hour, let endMinute = endComponents.minute else {
                print("Failed to extract hour and minute components.")
                return .failed
            }
            
            if currentHour < startHour || (currentHour == startHour && currentMinute < startMinute) {
                return .notStart
            } else if currentHour > endHour || (currentHour == endHour && currentMinute > endMinute) {
                return .end
            } else {
                return .inProgress
            }
        }
        else {
            return .none
        }
    }
    
    func navigationMethod (by shift: ShiftModel) {
        
        let shiftStart = shift.getStartTimeForToday()
        let shiftEnd = shift.getEndTimeForToday()
        
        print("shiftStart: \(shiftStart)")
        print("shiftEnds: \(shiftEnd)")
        
        let shiftStatus = checkShiftStatus(startsAt: shiftStart, endsAt: shiftEnd)
        
        switch shiftStatus {
        case .failed:
            print("time formate error")
        case .inProgress:
            let alert = AlertViewController()
            alert.shift = shift
            alert.shiftStatus = shiftStatus
            viewController.navigationController?.pushViewController(alert, animated: true)

        case .notStart, .end, .none, .weekend:
            let connect = ConnectViewController()
            connect.shift = shift
            connect.shiftStatus = shiftStatus
            viewController.navigationController?.pushViewController(connect, animated: true)
        }
    }
    
    private func setupTracking() {
        UIDevice.current.isBatteryMonitoringEnabled = true
        
        let userDefaults = UserDefaults.standard
        if userDefaults.string(forKey: "device_id_preference") == nil {
            let identifier = "\(Int.random(in: 100000..<1000000))"
            userDefaults.setValue(identifier, forKey: "device_id_preference")
        }
        
        AppDelegate.instance.registerDefaultsFromSettingsBundle()
        
        AppDelegate.instance.migrateLegacyDefaults()
        
        let modelUrl = Bundle.main.url(forResource: "TraccarClient", withExtension: "momd")
        AppDelegate.instance.managedObjectModel = NSManagedObjectModel(contentsOf: modelUrl!)
        
        AppDelegate.instance.persistentStoreCoordinator = NSPersistentStoreCoordinator(managedObjectModel: AppDelegate.instance.managedObjectModel!)
        let storeUrl = FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).last?.appendingPathComponent("TraccarClient.sqlite")
        let options = [NSMigratePersistentStoresAutomaticallyOption: true, NSInferMappingModelAutomaticallyOption: true]
        try! AppDelegate.instance.persistentStoreCoordinator?.addPersistentStore(ofType: NSSQLiteStoreType, configurationName: nil, at: storeUrl, options: options)
        
        AppDelegate.instance.managedObjectContext = NSManagedObjectContext.init(concurrencyType: .mainQueueConcurrencyType)
        AppDelegate.instance.managedObjectContext?.persistentStoreCoordinator = AppDelegate.instance.persistentStoreCoordinator
    }
    private func signinRequest(forcing: Bool) {
        var params: PARAMS = [:]
        
        for item in fields {
            if item.field == .domain {
                UserDefaults.standard.set("\(item.value.value).nmo.ai", forKey: "xcompany")
                UserDefaults.standard.set(fleetUrl, forKey: "server_url_preference")
            }
            if item.type == .text && item.field != .domain {
                params.add(item.field.rawValue, item.value.value)
            }
        }
        
        let spinner = viewController.spin()
        api.signin(params: params, force: forcing) { [weak self] result in
            guard let strongSelf = self else {return}
            if result.success {
                api.my_shift { result in
                    if result.success {
                        if let shift = result.data as? ShiftModel {
                            strongSelf.shift = shift
                            api.permissions { result in
                                if result.success {
                                    api.parameters { result in
                                        spinner.stop()
                                        if result.success {
                                            for item in strongSelf.fields {
                                                if item.field == .username {
                                                    UserDefaults.standard.set("\(item.value.value.lowercased())", forKey: "device_id_preference")
                                                    strongSelf.setupTracking()
                                                    strongSelf.navigationMethod(by: shift)
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    } else {
                        ok(result)
                    }
                }
            } else {
                spinner.stop()
                if result.statusCode == 401 {
                    do {
                        let value = try JSONSerialization.jsonObject(with: result.dataResponse?.data ?? Data())
                        if let dict = value as? JSON {
                            if let display_errors = dict["display_errors"] as? [String]  {
                                var displayError: String = ""
                                displayError = display_errors.joined(separator: "\n")
                                ok(displayError, viewController: self?.viewController)
                            }
                        }
                        
                    } catch {
                        print("Error while decoding response: \(String(data: result.dataResponse?.data ?? Data(), encoding: .utf8) ?? "")")
                    }
                } else if result.statusCode == 422 {
                    var fieldErrors: JSON = [:]
                    var errors: String = ""
                    do{
                        let value = try JSONSerialization.jsonObject(with: result.dataResponse?.data ?? Data())
                        if let dict = value as? JSON {
                            if let field_errors = dict["errors"] as? JSON {
                                fieldErrors = field_errors
                                strongSelf.fields.forEach { field in
                                    if let value = fieldErrors[field.field] as? [String] {
                                        value.forEach { string in
                                            errors = value.joined(separator: "\n")
                                            field.error.value = errors
                                        }
                                    }
                                }
                            }
                        }
                    } catch {
                        print("Error while decoding response: \(String(data: result.dataResponse?.data ?? Data(), encoding: .utf8) ?? "")")
                    }
                } else if result.statusCode == 404 {
                    ok("Domain is not valid!")
                }
                else if result.statusCode == 403 {
                    alert(confirm: "Continue here", destructive: "Cancel", message: result.displayError) {
                        strongSelf.viewController.dismiss(animated: true)
                        strongSelf.signinRequest(forcing: true)
                    }
                }
                
                else {
                    ok(result)
                }
            }
        }
    }
}

// MARK: - Request
extension LoginModelController {
    func signin() {
        signinRequest(forcing: false)
    }
}


// MARK: - Output Protocol
extension LoginModelController: LoginOutputProtocol {
    func cellButtonTapped(action: KNButtonViewAction) {
        if action == .tap {
            signin()
        }
        if action == .loginAsLeader {
            let leader = LeaderViewController()
            self.viewController.navigationController?.pushViewController(leader, animated: true)
        }
    }
    
    func reloadUI() {
        delegate?.updateUI()
    }
}


