//
//  ConnectViewController.swift
//  TraccarClient
//
//  Created by George Makhoul on 31/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit
//import InAppSettingsKit
import AppTrackingTransparency

final class ConnectViewController: KNViewController {
    // MARK: - Outlets
    @IBOutlet weak var customHeaderView: CustomHeaderView!
    @IBOutlet weak var connectButton: UIButton!
    @IBOutlet private weak var connectionStatusLogo: UIImageView!
    @IBOutlet private weak var discLabel: UILabel!
    @IBOutlet private weak var statusLabel: UILabel!
    @IBOutlet private weak var timerLabel: UILabel!
    @IBOutlet private weak var shiftsView: ShiftsView!
    @IBOutlet private weak var trackingView: TrackingView!
    @IBOutlet private weak var welcomeLabel: UILabel!
    
    // MARK: - Properties
    let userDefaults = UserDefaults.standard
    var trackingController: TrackingController?
    private lazy var totalTime = 0
    
    private var timer: Timer?
    private var isInBackground: Bool = false
    private var currentBackgroundDate = Date()
    let status: Bool = false
    var shift: ShiftModel?
    let timeFormat = "HH:mm"
    let dateFormatter = DateFormatter()
    var shiftStatus: shiftStatus = .failed
    
    // MARK: - LifeCycle
    override func viewDidLoad() {
        super.viewDidLoad()
        setupUI()
    }
    
    deinit {
        NotificationCenter.default.removeObserver(self, name: UIApplication.didEnterBackgroundNotification, object: nil)
        NotificationCenter.default.removeObserver(self, name: UIApplication.didBecomeActiveNotification, object: nil)
    }
    
    private func setupUI() {
        welcomeLabel.set(text: "Hello \(UserModel.loadFromUserDefaults().first_name)!", color: .secondary, font: .semibold(20))
        if #available(iOS 14, *) {
            ATTrackingManager.requestTrackingAuthorization { status in
                switch status {
                case .authorized:
                    print("Tracking Authorized.")
                    
                case .denied:
                    // Tracking authorization dialog was shown and permission was not granted.
                    print("Tracking denied.")
                case .notDetermined:
                    // Tracking authorization dialog has not been shown.
                    print("Tracking not determined.")
                case .restricted:
                    // The device is not eligible for tracking.
                    print("Tracking restricted.")
                @unknown default:
                    // A new case was added that we're not handling.
                    print("Unknown case.")
                }
            }
        } else {
            // don't do anything
        }
        
        setupShiftView()
        customHeaderView.set(type: .home, delegate: self)
        timerLabel.text = ""
        connectButton.set(title: "")
        
        discLabel.set(text: "We're currently tracking your journey. Drive safely!", color: .black, font: .medium(12))
        discLabel.textAlignment = .center
        let status = userDefaults.bool(forKey: "service_status_preference")
        statusChanged(to: status)
        
        NotificationCenter.default.addObserver(self, selector: #selector(appInBackground), name: UIApplication.didEnterBackgroundNotification, object: nil)
        
        NotificationCenter.default.addObserver(self, selector: #selector(appResumed), name: UIApplication.willEnterForegroundNotification, object: nil)
        
    }
    
    private func setupShiftView() {
        switch shiftStatus {
        case .inProgress:
            trackingView.isHidden = false
            shiftsView.isHidden = true
            setupTrackingView()
            
            startTimer()
            userDefaults.set(true, forKey: "service_status_preference")
            statusChanged(to: true)
            TransactionsViewController.addMessage(NSLocalizedString("Service created", comment: ""))
            AppDelegate.instance.trackingController = TrackingController()
            AppDelegate.instance.trackingController?.start()
            
        case .end, .notStart, .weekend:
            trackingView.isHidden = true
            shiftsView.isHidden = false
            shiftsView.set(model: shift ?? ShiftModel(), delegate: self)
            
        case .failed:
            print("error in time formate")
            
        case .none:
            shiftsView.isHidden = true
            trackingView.isHidden = true
            
        }
    }
    
    private func setupTrackingView() {
        trackingView.set(model: shift ?? ShiftModel(), delegate: self)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        hasCustomNavigation = true
        statusChanged(to: userDefaults.bool(forKey: "service_status_preference"))
    }
    
    
    private func statusChanged(to: Bool) {
        
        connectionStatusLogo.set(icon: to ? .connectedLogo : .disconnectedLogo)
        connectButton.set(icon: to ? .connectedButton : .disconnectedButton)
        statusLabel.set(text: to ? "You're on the map!" : "Tracking stopped.", color: .black, font: .bold(20))
    }
    
    // MARK: - Actions
    @IBAction func connectTapped(_ sender: UIButton) {
        if userDefaults.bool(forKey: "service_status_preference") {
            endTimer()
            userDefaults.set(false, forKey: "service_status_preference")
            statusChanged(to: false)
            TransactionsViewController.addMessage(NSLocalizedString("Service destroyed", comment: ""))
            AppDelegate.instance.trackingController?.stop()
            AppDelegate.instance.trackingController = nil
            
        } else {
            startTimer()
            userDefaults.set(true, forKey: "service_status_preference")
            statusChanged(to: true)
            TransactionsViewController.addMessage(NSLocalizedString("Service created", comment: ""))
            AppDelegate.instance.trackingController = TrackingController()
            AppDelegate.instance.trackingController?.start()
        }
        
    }
}

// MARK: - Timer Methods
extension ConnectViewController {
    func startTimer() {
        timerLabel.set(text: "", color: .lightGray, font: .semibold(14))
        
        timer = Timer.scheduledTimer(timeInterval: 1, target: self, selector: #selector(updateTime), userInfo: nil, repeats: true)
        
        RunLoop.current.add(timer!, forMode: RunLoop.Mode.common)
    }
    
    @objc func updateTime() {
        timeFormatted(totalTime)
        totalTime += 1
    }
    
    @objc private func appInBackground() {
        timer?.invalidate()
        currentBackgroundDate = Date()
    }
    
    @objc private func appResumed() {
        if userDefaults.bool(forKey: "service_status_preference") {
            let difference = currentBackgroundDate.timeIntervalSince(Date())
            print(currentBackgroundDate, difference, totalTime)
            
            let seconds = Int(abs(difference))
            totalTime += seconds
            
            timer = Timer.scheduledTimer(timeInterval: 1.0, target: self, selector: #selector(updateTime), userInfo: nil, repeats: true)
            timer?.fire()
        } else {
            timer?.invalidate()
            timerLabel.set(text: "")
            totalTime = 0
        }
    }
    
    func endTimer() {
        timer?.invalidate()
        timerLabel.set(text: "")
        totalTime = 0
    }
    
    func timeFormatted(_ totalSeconds: Int)  {
        let seconds: Int = totalSeconds % 60
        let minutes: Int = (totalSeconds / 60) % 60
        let timerText = "\(String(format: "%02d:%02d", minutes, seconds))"
        timerLabel.set(text: timerText, color: .lightGray, font: .semibold(14))
    }
}
extension ConnectViewController: CustomHeaderViewDelegate {
    func rightButtonTapped(action: RightButtonAction) {
        let transactions = TransactionsViewController()
        self.navigationController?.pushViewController(transactions, animated: true)
        
    }
    
    func leftButtonTapped(action: LeftButtonAction) {
        let settings = SettingsViewController()
        self.navigationController?.pushViewController(settings, animated: true)
    }
}

extension ConnectViewController: TrackingViewDelegate {
    func endtime() {
        endTimer()
        userDefaults.set(false, forKey: "service_status_preference")
        statusChanged(to: false)
        TransactionsViewController.addMessage(NSLocalizedString("Service destroyed", comment: ""))
        AppDelegate.instance.trackingController?.stop()
        AppDelegate.instance.trackingController = nil
        
        trackingView.isHidden = true
        shiftsView.isHidden = false
        shiftsView.set(model: shift ?? ShiftModel(), delegate: self)
    }
}

extension ConnectViewController: ShiftViewDelegate {
    func startTime() {
        startTimer()
        userDefaults.set(true, forKey: "service_status_preference")
        statusChanged(to: true)
        TransactionsViewController.addMessage(NSLocalizedString("Service created", comment: ""))
        AppDelegate.instance.trackingController = TrackingController()
        AppDelegate.instance.trackingController?.start()
        
        trackingView.isHidden = false
        shiftsView.isHidden = true
        setupTrackingView()
    }
}
