//
//  TrackingView.swift
//  TraccarClient
//
//  Created by George Makhoul on 21/11/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit
protocol TrackingViewDelegate {
    func endtime()
}

final class TrackingView: KNComponentView {
    // MARK: - Outlets
    @IBOutlet var view: UIView!
    @IBOutlet private weak var timeSlider: UISlider!
    @IBOutlet private weak var connectionStatusLabel: UILabel!
    @IBOutlet private weak var desc1Label: UILabel!
    @IBOutlet private weak var desk2Label: UILabel!
    @IBOutlet private weak var timeLeftLabel: UILabel!
    @IBOutlet private weak var startTimeView: UIView!
    @IBOutlet private weak var startTimeLabel: UILabel!
    @IBOutlet private weak var endTimeView: UIView!
    @IBOutlet private weak var endTimeLabel: UILabel!
    @IBOutlet private weak var startShiftLabel: UILabel!
    @IBOutlet private weak var endShiftLable: UILabel!
    @IBOutlet private weak var workingTimeLable: UILabel!
    
    // MARK: - Properties
    private var shift: ShiftModel?
    var currentTime: Date!
    var endTimeDate: Date!
    var timer: Timer!
    let timeFormat = "HH:mm"
    let interval = 60.0 // 1 minute
    var delegate: TrackingViewDelegate?
    
    // MARK: - Lifecycle
    override func setupNib() {
        super.setupNib()
        
        view.frame = bounds
        view.autoresizingMask = [.flexibleHeight, .flexibleWidth]
        
        setupUI()
    }
    
    private func setupUI() {
        timeSlider.setThumbImage(.handle, for: .normal)
        timeSlider.isUserInteractionEnabled = false
        
        connectionStatusLabel.set(text: "You're on the map!", color: .black, font: .bold(20))
        connectionStatusLabel.textAlignment = .center
        
        desc1Label.set(text: "Tracking will be working", color: .black, font: .regular(18))
        desc1Label.textAlignment = .center
        
        desk2Label.set(text: "until the end of your shift after", color: .black, font: .regular(18))
        desk2Label.textAlignment = .center
        
        timeLeftLabel.set(text: "3 hours 19 minutes", color: .black, font: .bold(18))
        timeLeftLabel.textAlignment = .center
        
        startTimeView.addCorners(4)
        startTimeView.backgroundColor = .invalid
        
        endTimeView.addCorners(4)
        endTimeView.backgroundColor = .invalid
        
        startTimeLabel.set(text: "09:00", color: .invalidText, font: .semibold(13))
        startTimeLabel.textAlignment = .center
        
        endTimeLabel.set(text: "17:00", color: .invalidText, font: .semibold(13))
        endTimeLabel.textAlignment = .center
        
        startShiftLabel.set(text: "Shift Starts", color: .invalid, font: .regular(11))
        startShiftLabel.textAlignment = .center
        
        endShiftLable.set(text: "Shift Ends", color: .invalid, font: .regular(11))
        
        //                workingTimeLable.set(text: "Working Time: 03:24", color: .black, font: .medium(10))
        workingTimeLable.set(text: "", color: .black, font: .medium(10))
        
    }
    
    func set(model: ShiftModel, delegate: TrackingViewDelegate) {
        self.delegate = delegate
        shift = model
        configureSlider(model: model)
        setupTimer()
    }
    
    func configureSlider(model: ShiftModel) {
        self.startTimeLabel.text = model.getStartTimeForToday()
        self.endTimeLabel.text = model.getEndTimeForToday()
        
        let dateFormatter = DateFormatter()
        dateFormatter.dateFormat = timeFormat
        
        guard let startDate = dateFormatter.date(from: model.getStartTimeForToday()),
              let endDate = dateFormatter.date(from: model.getEndTimeForToday()) else {
            return
        }
        
        currentTime = startDate
        endTimeDate = endDate
        
        timeSlider.minimumValue = 0
        timeSlider.maximumValue = Float(endDate.timeIntervalSince(startDate) / interval)
                
        let now = Date()
        print("now: \(now)")
        
        let calendar = Calendar.current
        let nowComponents = calendar.dateComponents([.year, .month, .day], from: now)
        
        var start: Date = currentTime
        var end: Date = endTimeDate
        
        var startComponents = calendar.dateComponents([.hour, .minute, .second], from: startDate)
        startComponents.year = nowComponents.year
        startComponents.month = nowComponents.month
        startComponents.day = nowComponents.day
        start = calendar.date(from: startComponents)!
        
        var endComponents = calendar.dateComponents([.hour, .minute, .second], from: endDate)
        endComponents.year = nowComponents.year
        endComponents.month = nowComponents.month
        endComponents.day = nowComponents.day
        end = calendar.date(from: endComponents)!
        
        if let endDate = Calendar.current.date(from: endComponents) {
            timeLeftLabel.text = calculateRemainingTime(endDate: endDate)
        }
        
        if now >= start && now <= end {
            timeSlider.value = Float(now.timeIntervalSince(start) / interval)
        } else {
            timeSlider.value = 0
        }
    }
    
    func setupTimer() {
        timer = Timer.scheduledTimer(timeInterval: interval, target: self, selector: #selector(updateSlider), userInfo: nil, repeats: true)
    }
    
    @objc func updateSlider() {
        timeSlider.value += 1
        
        guard let currentValue = currentTime else {
            return
        }
        
        let updatedTime = currentValue.addingTimeInterval(interval)
        
        let calendar = Calendar.current
        var endComponents = calendar.dateComponents([.hour, .minute, .second], from: endTimeDate)
        endComponents.year = calendar.component(.year, from: updatedTime)
        endComponents.month = calendar.component(.month, from: updatedTime)
        endComponents.day = calendar.component(.day, from: updatedTime)
        if currentTime > endTimeDate {
            // If start time is later than end time, the shift ends the next day
            endComponents.day = calendar.component(.day, from: updatedTime) + 1
        }
        let adjustedEndTimeDate = calendar.date(from: endComponents)!
        
        if updatedTime <= adjustedEndTimeDate {
            currentTime = updatedTime
            
            let dateFormatter = DateFormatter()
            dateFormatter.dateFormat = timeFormat
            
            let timeString = dateFormatter.string(from: updatedTime)
            print(timeString) // You can use this time string as per your requirement
            
            let now = Date()
            let calendar = Calendar.current
            let nowComponents = calendar.dateComponents([.year, .month, .day], from: now)
        
            var end: Date = endTimeDate
            print(end)
            
            guard let endDate = dateFormatter.date(from: shift?.getEndTimeForToday() ?? "") else {
                return
            }
            var endComponents = calendar.dateComponents([.hour, .minute, .second], from: endDate)
            endComponents.year = nowComponents.year
            endComponents.month = nowComponents.month
            endComponents.day = nowComponents.day
            end = calendar.date(from: endComponents)!
            
            if let endDate = Calendar.current.date(from: endComponents) {
                if calculateRemainingTime(endDate: endDate) != "Time's up!" {
                    timeLeftLabel.text = calculateRemainingTime(endDate: endDate)
                } else {
                    timer.invalidate()
                    delegate?.endtime()
                }
            }
        }
    }
    
    func calculateRemainingTime(endDate: Date) -> String {
        let remainingTimeInSeconds = endDate.timeIntervalSinceNow
        if remainingTimeInSeconds > 0 {
            let formatter = DateComponentsFormatter()
            formatter.allowedUnits = [.hour, .minute]
            formatter.unitsStyle = .full
            return formatter.string(from: TimeInterval(remainingTimeInSeconds)) ?? "Time's up!"
        } else {
            return "Time's up!"
        }
    }
}
