//
//  NetworkRequest.swift
//  traccar
//
//  Created by George Makhoul on 10/25/23.
//

import Foundation
import Alamofire
import UIKit

let api = NetworkRequest.si

struct RequestOptions {
    let authorization: Bool
    let validation: Bool
}

final class NetworkRequest: NSObject {
    
    // MARK: - Properties
    static let si = NetworkRequest()
    fileprivate let requestRetryLimit = 3
    
    // MARK: - Init
    override private init() {
        super.init()
    }
    
    // MARK: - HTTP Headers
    var headers : HTTPHeaders {
        var _headers: HTTPHeaders = [:]
        _headers["Content-Type"] = "application/json"
        _headers["Accept"] = "application/json"
        _headers["Accept-Language"] = "en"
        _headers["X-App-Client"] = "ios"
        _headers["X-Company"] = "\(UserDefaults.standard.string(forKey: "xcompany") ?? "")"
        return _headers
    }
    
    // MARK: - Endpoint
    func endpoint(_ endpoint: Endpoint, id: String = "") -> URLConvertible {
        let fullUrl: String = Endpoint.baseURL + endpoint.rawValue + id
        
        do {
            let regex = try NSRegularExpression(pattern: "( )*$", options: NSRegularExpression.Options.caseInsensitive)
            let range = NSMakeRange(0, fullUrl.count)
            return regex.stringByReplacingMatches(in: fullUrl, options: [], range: range, withTemplate: "") as URLConvertible
        } catch {
            return fullUrl as URLConvertible
        }
    }
    
    func stringUrl(_ endpoint: String) -> URLConvertible {
        let fullUrl: String = Endpoint.baseURL + endpoint
        
        do {
            let regex = try NSRegularExpression(pattern: "( )*$", options: NSRegularExpression.Options.caseInsensitive)
            let range = NSMakeRange(0, fullUrl.count)
            return regex.stringByReplacingMatches(in: fullUrl, options: [], range: range, withTemplate: "") as URLConvertible
        } catch {
            return fullUrl as URLConvertible
        }
    }
    
    func completeStringUrl(_ endpoint: String) -> URLConvertible {
        let fullUrl: String = endpoint
        
        do {
            let regex = try NSRegularExpression(pattern: "( )*$", options: NSRegularExpression.Options.caseInsensitive)
            let range = NSMakeRange(0, fullUrl.count)
            return regex.stringByReplacingMatches(in: fullUrl, options: [], range: range, withTemplate: "") as URLConvertible
        } catch {
            return fullUrl as URLConvertible
        }
    }
    
    func getQueryStringParameter(url: String, param: String) -> String? {
        guard let url = URLComponents(string: url) else { return nil }
        return url.queryItems?.first(where: { $0.name == param })?.value
    }
    
    func request(endpoint: URLConvertible, method: HTTPMethod, _ completion: @escaping ResultClosure) {
        
        if !isConnected {
            completion(NetworkResultModel(error: NetworkError(.noInternet)))
            return
        }
        
        var customValidation: [Int] = []
        customValidation.append(contentsOf: 1...400)
        customValidation.append(contentsOf: 402...999)
        
        AF.request(endpoint,
                   method: method,
                   parameters: nil,
                   encoding: JSONEncoding.default,
                   headers: headers,
                   interceptor: self).validate(statusCode: customValidation).responseData { response in
            completion(NetworkResultModel(dataResponse: response))
        }
    }
    func request(endpoint: URLConvertible, method: HTTPMethod, params: Parameters? = nil, _ completion: @escaping ResultClosure) {
        
        if !isConnected {
            completion(NetworkResultModel(error: NetworkError(.noInternet)))
            return
        }
        
        var customValidation: [Int] = []
        customValidation.append(contentsOf: 1...400)
        customValidation.append(contentsOf: 402...999)
        
        AF.request(endpoint,
                   method: method,
                   parameters: params,
                   encoding: JSONEncoding.default,
                   headers: headers,
                   interceptor: self).validate(statusCode: customValidation).responseData { response in
            completion(NetworkResultModel(dataResponse: response))
        }
    }
    
    func request(endpoint: URLConvertible, method: HTTPMethod, params: Parameters? = nil, options: RequestOptions? = nil, _ completion: @escaping ResultClosure) {
        
        if !isConnected {
            completion(NetworkResultModel(error: NetworkError(.noInternet)))
            return
        }
        
        var customValidation: [Int] = []
        customValidation.append(contentsOf: 1...400)
        customValidation.append(contentsOf: 402...999)
        
        if let options = options {
            let request = AF.request(endpoint,
                                     method: method,
                                     parameters: params,
                                     encoding: JSONEncoding.default,
                                     headers: options.authorization ? headers : nil,
                                     interceptor: self)
            if options.validation {
                request.validate(statusCode: customValidation)
            }
            request.responseData { response in
                completion(NetworkResultModel(dataResponse: response))
            }
        } else {
            AF.request(endpoint,
                       method: method,
                       parameters: params,
                       encoding: JSONEncoding.default,
                       headers: headers,
                       interceptor: self).responseData { response in
                completion(NetworkResultModel(dataResponse: response))
            }
        }
    }
    
    func requestToUpload(_ imageToUpload: UIImage, endpoint: URLConvertible, params: Parameters? = nil, options: RequestOptions? = nil, _ completion: @escaping ResultClosure){
        
        if !isConnected {
            completion(NetworkResultModel(error: NetworkError(.noInternet)))
            return
        }
        
        var customValidation: [Int] = []
        customValidation.append(contentsOf: 1...400)
        customValidation.append(contentsOf: 402...999)
        
        guard let imageData = imageToUpload.jpegData(compressionQuality: 0.50) else { return }
        
        let  timeStamp = Int(Date().timeIntervalSince1970 * 1000)
        
        if let options = options {
            let request = AF.upload(multipartFormData: { (multipartFormData) in
                multipartFormData.append(imageData, withName: "file", fileName: "file\(timeStamp).jpeg", mimeType: "image/jpeg")
            }, to: endpoint, method: .post , headers: headers)
            
            if options.validation {
                request.validate(statusCode: customValidation)
            }
            request.responseData { response in
                completion(NetworkResultModel(dataResponse: response))
            }
        } else {
            
            AF.upload(multipartFormData: { (multipartFormData) in
                multipartFormData.append(imageData, withName: "file", fileName: "file\(timeStamp).jpeg", mimeType: "image/jpeg")
            }, to: endpoint, method: .post , headers: headers).responseData { response in
                completion(NetworkResultModel(dataResponse: response))
            }
        }
    }
    
    func requestToUploadVideo(_ videoToUpload: URL, endpoint: URLConvertible, params: Parameters? = nil, options: RequestOptions? = nil, _ completion: @escaping ResultClosure){
        
        if !isConnected {
            completion(NetworkResultModel(error: NetworkError(.noInternet)))
            return
        }
        
        var customValidation: [Int] = []
        customValidation.append(contentsOf: 1...400)
        customValidation.append(contentsOf: 402...999)
        
        let  timeStamp = Int(Date().timeIntervalSince1970 * 1000)
        
        if let options = options {
            let request = AF.upload(multipartFormData: { (multipartFormData) in
                multipartFormData.append(videoToUpload, withName: "file", fileName: "file\(timeStamp).mp4", mimeType: "\(timeStamp)/mp4")
            }, to: endpoint, method: .post , headers: headers)
            
            if options.validation {
                request.validate(statusCode: customValidation)
            }
            request.responseData { response in
                completion(NetworkResultModel(dataResponse: response))
            }
        } else {
            
            AF.upload(multipartFormData: { (multipartFormData) in
                multipartFormData.append(videoToUpload, withName: "file", fileName: "file\(timeStamp).mp4", mimeType: "\(timeStamp)/mp4")
            }, to: endpoint, method: .post , headers: headers).responseData { response in
                completion(NetworkResultModel(dataResponse: response))
            }
        }
    }
    
    func requestToDownload(downloadURL: String, completion: @escaping (_ isDownloaded: Bool, _ downloadPath: String?) -> Void ){
        
        if !isConnected {
            //            completion(NetworkResultModel(error: NetworkError(.noInternet)))
            return
        }
        
        var customValidation: [Int] = []
        customValidation.append(contentsOf: 1...400)
        customValidation.append(contentsOf: 402...999)
        
        guard let documentsURL = FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).first else {
            return completion(false, nil)
        }
        
        let destination: DownloadRequest.Destination = { _, _ in
            let timeStamp = Int(Date().timeIntervalSince1970 * 1000)
            
            let fileName = (downloadURL.components(separatedBy: "/").last)?.components(separatedBy: ".")
            
            let name = fileName?.first ?? "\(timeStamp)"
            let format = fileName?.last ?? ""
            
            let fileURL = documentsURL.appendingPathComponent("traccar-\(name).\(format)")
            
            return (fileURL, [.removePreviousFile, .createIntermediateDirectories])
        }
        
        AF.download(downloadURL, to: destination).response { response in
            
            if response.error == nil, let filePath = response.fileURL?.path {
                completion(true, filePath)
                
            } else {
                completion(false, nil)
            }
        }
    }
}

// MARK: - Helpers
extension NetworkRequest {
    
    // MARK: - Internet Connection
    var isConnected: Bool {
        return ReachabilityManager.sharedInstance.isConnectedToNetwork()
    }
}

// MARK: - Interceptor
extension NetworkRequest: RequestInterceptor {
    public func adapt(_ urlRequest: URLRequest, for session: Session, completion: @escaping (Result<URLRequest, Error>) -> Void) {
        
        guard let access_token = UserDefaults.standard.string(forKey: Keys.User.access_token), access_token != "" else {
            
            guard let guestToken = UserDefaults.standard.string(forKey: Keys.guestToken), guestToken != "" else {
                
                completion(.success(urlRequest))
                return
            }
            
            var request = urlRequest
            let bearer_token = "Bearer \(guestToken)"
            request.setValue(bearer_token, forHTTPHeaderField: "Authorization")
            
            completion(.success(request))
            return
        }
        
        var request = urlRequest
        let bearer_token = "Bearer \(access_token)"
        request.setValue(bearer_token, forHTTPHeaderField: "Authorization")
        
        //log("@NetworkRequest: RequestInterceptor @adapt", "ACCESS_TOKEN", UserDefaults.standard.string(forKey: Keys.User.access_token) as Any, "REFRESH_TOKEN", UserDefaults.standard.string(forKey: Keys.User.refresh_token) as Any)
        
        completion(.success(request))
    }
    
    public func retry(_ request: Request, for session: Session, dueTo error: Error, completion: @escaping (RetryResult) -> Void) {
        
        guard let response = request.response else {
            completion(.doNotRetry)
            return
        }
        
        let statusCode = response.statusCode
        
        switch statusCode {
        case 200...400:
            completion(.doNotRetry)
        case 401:
            refreshAccessToken { success in
                if success {
                    completion(.retry)
                } else {
                    completion(.doNotRetry)
                }
            }
        case 402...999:
            completion(.doNotRetry)
        default:
            if request.retryCount < requestRetryLimit {
                completion(.retry)
                return
            }
        }
    }
    
    func refreshAccessToken(completion: @escaping (_ success: Bool) -> Void) {
        guard let _ = UserDefaults.standard.string(forKey: Keys.User.refresh_token) else {
            completion(false)
            return
        }
        
        //        let params = ["refresh_token" : refresh_token]
        //
        //        api.refreshToken(params: params) { result in
        //            if result.success {
        //                if let newAccessToken = result.data as? String {
        //                    UserDefaults.standard.setValue(newAccessToken, forKey: Keys.User.access_token)
        //                    completion(true)
        //                } else {
        //                    completion(false)
        //                }
        //            } else {
        //                guard let error = result.appError else {
        //                    completion(false)
        //                    return
        //                }
        //
        //                if error.type == .invalidRefreshToken {
        //                    app.delegate.presentLoginViewController(with: error.message)
        //                    completion(false)
        //                } else if error.type == .invalidAccessToken {
        //                    completion(false)
        //                }
        //            }
        //        }
    }
}

