//
//  Observable.swift
//  TraccarClient
//
//  Created by George Makhoul on 25/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import Foundation

final class Observable<T> {
    typealias ObserverClosure = (T) -> Void
    
    private(set) var observers: [ObjectIdentifier: ObserverClosure] = [:]
    
    var value: T {
        didSet {
            notifyObservers()
        }
    }
    
    init(_ value: T) {
        self.value = value
    }
    
    func notifyObservers() {
        for (_, observer) in observers {
            observer(value)
        }
    }
    
    func observe(_ observer: @escaping ObserverClosure) -> Disposable {
        let uniqueKey: ObjectIdentifier = ObjectIdentifier(self)
        
        observers[uniqueKey] = observer
        
        return ObserverDisposable(owner: self, key: uniqueKey)
    }
    
    func removeObserver(with key: ObjectIdentifier) {
        if observers.keys.contains(key) {
            observers.removeValue(forKey: key)
            notifyObservers()
        }
    }
}


final class ObserverDisposable<T>: Disposable {
    var key: ObjectIdentifier
    weak var owner: Observable<T>?
    
    init(owner: Observable<T>, key: ObjectIdentifier) {
        self.owner = owner
        self.key = key
    }
    
    func dispose() {
        self.owner?.removeObserver(with: key)
    }
}

protocol Disposable {
    func dispose()
}

extension Disposable {
    func disposed(by bag: DisposeBag) {
        bag.add(self)
    }
}


final class DisposeBag {
    var disposables: [Disposable] = []
    
    func add(_ disposable: Disposable) {
        disposables.append(disposable)
    }
    
    func dispose() {
        disposables.forEach {
            $0.dispose()
        }
    }
    
    deinit {
        dispose()
    }
}



