//
//  SettingsViewController.swift
//  TraccarClient
//
//  Created by George Makhoul on 31/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit
import InAppSettingsKit

final class SettingsViewController: KNViewController {
    // MARK: - Outlets
    @IBOutlet weak var customHeaderView: CustomHeaderView!
    @IBOutlet weak var tableView: UITableView!
    
    // MARK: - Properties
    private var textDefaults: [DefaultsModel] = []
    
    private var switchDefaults: [DefaultsModel] = []
    
    private var multiChoicesDefaults: [DefaultsModel] = []
    
    private var editedTextDefaults: [DefaultsModel] = []
    private var editedSwitchDefaults: [DefaultsModel] = []
    private var editedMultiChoicesDefaults: [DefaultsModel] = []
    
    
    var trackingController: TrackingController?
    
    // MARK: - LifeCycle
    override func viewDidLoad() {
        super.viewDidLoad()
        setupUI()
    }
    
    private func setupUI() {
        setDefaults()
        customHeaderView.set(type: .settings, delegate: self)
        
        tableView.delegate = self
        tableView.dataSource = self
        
        tableView.register(cells: [SettingsTextTableViewCell.self,
                                   SettingsSwitchTableViewCell.self,
                                   SettingsSegmentTableViewCell.self,
                                   SettingsLogoutTableViewCell.self,
                                   SettingsPrivacyPolicyTableViewCell.self,
                                   SettingsResetTableViewCell.self])
        
        tableView.allowsSelection = false
        tableView.separatorStyle = .none
    }
    
    private func setDefaults() {
        textDefaults = [DefaultsModel(title: "Frequency",
                                      value: UserDefaults.standard.string(forKey: "frequency_preference") ?? "",
                                      keyboardType: .numberPad,
                                      key: "frequency_preference"),
                        
                        DefaultsModel(title: "Distance",
                                      value: UserDefaults.standard.string(forKey: "distance_preference") ?? "",
                                      keyboardType: .numberPad,
                                      key: "distance_preference"),
                        
                        DefaultsModel(title: "Angle",
                                      value: UserDefaults.standard.string(forKey: "angle_preference") ?? "",
                                      keyboardType: .numberPad,
                                      key: "angle_preference")]
        
        switchDefaults = [DefaultsModel(title: "Offline buffering",
                                        value: UserDefaults.standard.bool(forKey: "buffer_preference") ,
                                        keyboardType: .default,
                                        key: "buffer_preference")]
        
        multiChoicesDefaults = [DefaultsModel(title: "Accuracy",
                                              value: UserDefaults.standard.string(forKey: "accuracy_preference") ?? "",
                                              keyboardType: .default,
                                              key: "accuracy_preference")]
    }
    
    private func isChangesOccured() -> Bool{
        for defaultItem in textDefaults {
            if let editedItem = editedTextDefaults.first(where: { $0.key == defaultItem.key }) {
                if "\(editedItem.value ?? "")" != "\(defaultItem.value ?? "")" {
                    return true
                }
            }
        }
        for defaultItem in switchDefaults {
            if let editedItem = editedSwitchDefaults.first(where: { $0.key == defaultItem.key }) {
                if "\(editedItem.value ?? "")" != "\(defaultItem.value ?? "")" {
                    return true
                }
            }
        }
        for defaultItem in multiChoicesDefaults {
            if let editedItem = editedMultiChoicesDefaults.first(where: { $0.key == defaultItem.key }) {
                if "\(editedItem.value ?? "")" != "\(defaultItem.value ?? "")" {
                    return true
                }
            }
        }
        return false
    }
}

extension SettingsViewController: CustomHeaderViewDelegate {
    func rightButtonTapped(action: RightButtonAction) {
        view.endEditing(true)
        if isChangesOccured() {
            alert(confirm: "Yes", destructive: "No", message: "Are you sure you want to change settings?") {
                UserDefaults.standard.setValue(true, forKey: "params_changed")
                for item in self.editedTextDefaults {
                    UserDefaults.standard.setValue(item.value, forKey: item.key)
                }
                for item in self.editedSwitchDefaults {
                    UserDefaults.standard.setValue(item.value, forKey: item.key)
                }
                for item in self.editedMultiChoicesDefaults {
                    UserDefaults.standard.setValue(item.value, forKey: item.key)
                }
                if AppDelegate.instance.trackingController != nil {
                    
                    TransactionsViewController.addMessage(NSLocalizedString("Settings Changed", comment: ""))
                    AppDelegate.instance.trackingController?.stop()
                    AppDelegate.instance.trackingController = nil
                    
                    AppDelegate.instance.trackingController = TrackingController()
                    AppDelegate.instance.trackingController?.start()
                }
                self.dismiss(animated: true)
                self.navigationController?.popViewController(animated: true)
            }
        } else {
            self.dismiss(animated: true)
            self.navigationController?.popViewController(animated: true)
        }
    }
    
    func leftButtonTapped(action: LeftButtonAction) {
        view.endEditing(true)
        if isChangesOccured() {
            alert(presentingBy: self, confirm: "Save Changes", destructive: "Discard Changes", message: "You have unsaved changes. What do you want to do?") {
                UserDefaults.standard.setValue(true, forKey: "params_changed")
                for item in self.editedTextDefaults {
                    UserDefaults.standard.setValue(item.value, forKey: item.key)
                }
                
                for item in self.editedSwitchDefaults {
                    UserDefaults.standard.setValue(item.value, forKey: item.key)
                }
                
                for item in self.editedMultiChoicesDefaults {
                    UserDefaults.standard.setValue(item.value, forKey: item.key)
                }
                
                if AppDelegate.instance.trackingController != nil {
                    TransactionsViewController.addMessage(NSLocalizedString("Settings Changed", comment: ""))
                    AppDelegate.instance.trackingController?.stop()
                    AppDelegate.instance.trackingController = nil
                    
                    AppDelegate.instance.trackingController = TrackingController()
                    AppDelegate.instance.trackingController?.start()
                }
                
                self.dismiss(animated: true)
                self.navigationController?.popViewController(animated: true)
                
            } _: {
                self.dismiss(animated: true)
                self.navigationController?.popViewController(animated: true)
            }
            
        } else {
            self.navigationController?.popViewController(animated: true)
        }
    }
    
    func logoutRequest() {
        let spinner = spin()
        api.logout { result in
            spinner.stop()
            if result.success {
                if let navigation = self.navigationController {
                    
                    UserModel.clearUserFromUserDefaults()
                    DeviceModel.clearUserFromUserDefaults()
                    SignInModel.clearUserFromUserDefaults()
                    
                    let coordinator = AuthenticationCoordinator(navigationController: navigation)
                    coordinator.step = .signin
                    coordinator.start()
                }
                
            } else {
                ok(result)
            }
        }
    }
}

// MARK: - UITableViewDelegate
extension SettingsViewController: UITableViewDelegate, UITableViewDataSource {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        UserDefaults.standard.bool(forKey: Keys.User.updateSettings) ? 5 : 1
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if UserDefaults.standard.bool(forKey: Keys.User.updateSettings) {
            switch section {
            case 0:
                return textDefaults.count
                
            case 1:
                return multiChoicesDefaults.count
                
            case 2:
                return switchDefaults.count
            case 3:
                return 1
            case 4:
                return 2
                
            default:
                return 0
            }
        } else {
            return 2
        }
    }
    
    func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        if UserDefaults.standard.bool(forKey: Keys.User.updateSettings) {
            if section == 0 {
                return "Tracking Parameters"
            }
            
            if section == 4 {
                return "Settings"
            }
        }
        
        return ""
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if UserDefaults.standard.bool(forKey: Keys.User.updateSettings) {
            if indexPath.section == 0 {
                let cell = tableView.dequeue(withCell: SettingsTextTableViewCell.self)
                cell.set(defaultValue: textDefaults[indexPath.row], index: indexPath.row)
                cell.delegate = self
                
                return cell
            }
            
            if indexPath.section == 1 {
                let cell = tableView.dequeue(withCell: SettingsSegmentTableViewCell.self)
                cell.set(defaultValue: multiChoicesDefaults[indexPath.row], index: indexPath.row)
                cell.delegate = self
                return cell
            }
            
            if indexPath.section == 2 {
                let cell = tableView.dequeue(withCell: SettingsSwitchTableViewCell.self)
                cell.set(defaultValue: switchDefaults[indexPath.row], index: indexPath.row)
                cell.delegate = self
                return cell
            }
            
            if indexPath.section == 3 {
                let cell = tableView.dequeue(withCell: SettingsResetTableViewCell.self)
                cell.set(title: "Reset")
                cell.delegate = self
                return cell
            }
            
            if indexPath.section == 4 {
                if indexPath.row == 1 {
                    let cell = tableView.dequeue(withCell: SettingsLogoutTableViewCell.self)
                    cell.set(title: "Logout")
                    cell.delegate = self
                    return cell
                }
                if indexPath.row == 0 {
                    let privacyCell = tableView.dequeue(withCell: SettingsPrivacyPolicyTableViewCell.self)
                    privacyCell.set(title: "Privacy Policy")
                    privacyCell.delegate = self
                    return privacyCell
                }
            }
        }
        else {
            if indexPath.row == 1 {
                let cell = tableView.dequeue(withCell: SettingsLogoutTableViewCell.self)
                cell.set(title: "Logout")
                cell.delegate = self
                return cell
            }
            if indexPath.row == 0 {
                let privacyCell = tableView.dequeue(withCell: SettingsPrivacyPolicyTableViewCell.self)
                privacyCell.set(title: "Privacy Policy")
                privacyCell.delegate = self
                return privacyCell
            }
        }
        return UITableViewCell()
    }
}

func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
    70
}


// MARK: - CellsDelegate
extension SettingsViewController: SettingsTextTableViewCellDelegate,
                                  SettingsSwitchTableViewCellDelegate,
                                  SettingsSegmentTableViewCellDelegate {
    func didUpdateText(model: DefaultsModel) {
        editedTextDefaults.append(model)
    }
    
    
    func didUpdateSwitch(model: DefaultsModel) {
        editedSwitchDefaults.append(model)
    }
    
    func didUpdateSegment(model: DefaultsModel) {
        editedMultiChoicesDefaults.append(model)
    }
}

// MARK: - SettingsLogoutTableViewCellDelegate
extension SettingsViewController: SettingsLogoutTableViewCellDelegate {
    func logout() {
        if AppDelegate.instance.trackingController != nil {
            
            alert(presentingBy: self, confirm: "Yes" , destructive: "Cancel", message: "It seems that you are on the map, do you want to stop the location update before logging out?") {
                UserDefaults.standard.set(false, forKey: "service_status_preference")
                TransactionsViewController.addMessage(NSLocalizedString("Service destroyed", comment: ""))
                AppDelegate.instance.trackingController?.stop()
                AppDelegate.instance.trackingController = nil
                self.dismiss(animated: true)
                self.logoutRequest()
            }
            
        } else {
            alert(confirm: "Yes", destructive: "No", message: "Are you sure you want to logout?") {
                self.dismiss(animated: true)
                self.logoutRequest()
            }
        }
    }
}

// MARK: - SettingsPrivacyTableViewCellDelegate
extension SettingsViewController: SettingsPrivacyTableViewCellDelegate {
    func showPrivacy() {
        guard let url = URL(string: "https://dev-nextcloud.nmo.ai/apps/teams/img/privacy-policy.html") else {
            return //be safe
        }
        
        if #available(iOS 10.0, *) {
            UIApplication.shared.open(url, options: [:], completionHandler: nil)
        } else {
            UIApplication.shared.openURL(url)
        }
    }
}

// MARK: - SettingsResetTableViewCellDelegate
extension SettingsViewController: SettingsResetTableViewCellDelegate {
    func reset() {
        alert(presentingBy: self, confirm: "Yes" , destructive: "Cancel", message: "Are you sure you want to reset tracking parameters?") {
            self.dismiss(animated: true)
            UserDefaults.standard.setValue(false, forKey: "params_changed")
            let spinner = self.spin()
            api.parameters { result in
                spinner.stop()
                if result.success {
                    self.setDefaults()
                    self.tableView.reloadData()
                } else {
                    ok(result)
                }
            }
        }
    }
}
