//
//  UILabel.swift
//  TraccarClient
//
//  Created by George Makhoul on 25/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit

extension UILabel {
    // MARK: - Set Text, Color, Font
    func set(text: String) {
        self.text = text
        alignment()
    }
    
    func set(color: UIColor) {
        textColor = color
    }
    
    func set(font: UIFont) {
        self.font = font
    }
    
    func set(text: String, color: UIColor, font: UIFont) {
        set(text: text)
        set(color: color)
        set(font: font)
    }
    
    func set(uppercased: String) {
        text = uppercased.uppercased()
        alignment()
    }
    
    func set(uppercased: String, color: UIColor, font: UIFont) {
        set(uppercased: uppercased)
        set(font: font)
        set(color: color)
    }
    
    // MARK: - Localization
    func set(localized: String) {
        text = localized
        alignment()
    }
    
    func set(localizedUppercased: String) {
        text = localizedUppercased.uppercased()
        alignment()
    }
    
    func set(localized: String, color: UIColor, font: UIFont) {
        set(localized: localized)
        set(font: font)
        set(color: color)
    }
    
    func set(localizedUppercased: String, color: UIColor, font: UIFont) {
        set(localizedUppercased: localizedUppercased)
        set(font: font)
        set(color: color)
    }
    
    // MARK: - Alignment
    func alignment() {
        textAlignment = .left
    }
    
    func align(to alignment: NSTextAlignment) {
        textAlignment = alignment
    }
}

// MARK: - Helpers
extension UILabel {
    var maxNumberOfLines: Int {
        let maxSize = CGSize(width: frame.size.width, height: CGFloat(MAXFLOAT))
        let text = (self.text ?? "") as NSString
        let lineHeight = ceil(font.lineHeight)
        let textHeight = text.boundingRect(with: maxSize, options: .usesLineFragmentOrigin, attributes: [.font: font as UIFont], context: nil).height
        
        return Int(ceil(textHeight / lineHeight))
    }
    func countLines() -> Int {
        guard let myText = self.text as NSString? else {
            return 0
        }
        self.layoutIfNeeded()
        let rect = CGSize(width: self.bounds.width, height: CGFloat.greatestFiniteMagnitude)
        let labelSize = myText.boundingRect(with: rect, options: .usesLineFragmentOrigin, attributes: [NSAttributedString.Key.font: self.font as Any], context: nil)
        return Int(ceil(CGFloat(labelSize.height) / self.font.lineHeight))
    }
    
    var labelTextHeight: CGFloat {
        if text?.isEmpty ?? false{
            return 0.0
        }
        let labelTextSize = ((text ?? "") as NSString).boundingRect(
            with: CGSize(width: frame.size.width, height: .greatestFiniteMagnitude),
            options: .usesLineFragmentOrigin,
            attributes: [.font: font!],
            context: nil).size
        
        return labelTextSize.height
    }
    
}

extension UITapGestureRecognizer {

    func didTapAttributedTextInLabel(label: UILabel, inRange targetRange: NSRange) -> Bool {
        // Create instances of NSLayoutManager, NSTextContainer and NSTextStorage
        let layoutManager = NSLayoutManager()
        let textContainer = NSTextContainer(size: CGSize.zero)
        let textStorage = NSTextStorage(attributedString: label.attributedText!)

        // Configure layoutManager and textStorage
        layoutManager.addTextContainer(textContainer)
        textStorage.addLayoutManager(layoutManager)

        // Configure textContainer
        textContainer.lineFragmentPadding = 0.0
        textContainer.lineBreakMode = label.lineBreakMode
        textContainer.maximumNumberOfLines = label.numberOfLines
        let labelSize = label.bounds.size
        textContainer.size = labelSize

        // Find the tapped character location and compare it to the specified range
        let locationOfTouchInLabel = self.location(in: label)
        let textBoundingBox = layoutManager.usedRect(for: textContainer)
        //let textContainerOffset = CGPointMake((labelSize.width - textBoundingBox.size.width) * 0.5 - textBoundingBox.origin.x,
        //(labelSize.height - textBoundingBox.size.height) * 0.5 - textBoundingBox.origin.y);
        let textContainerOffset = CGPoint(x: (labelSize.width - textBoundingBox.size.width) * 0.5 - textBoundingBox.origin.x, y: (labelSize.height - textBoundingBox.size.height) * 0.5 - textBoundingBox.origin.y)

        //let locationOfTouchInTextContainer = CGPointMake(locationOfTouchInLabel.x - textContainerOffset.x,
        // locationOfTouchInLabel.y - textContainerOffset.y);
        let locationOfTouchInTextContainer = CGPoint(x: locationOfTouchInLabel.x - textContainerOffset.x, y: locationOfTouchInLabel.y - textContainerOffset.y)
        let indexOfCharacter = layoutManager.characterIndex(for: locationOfTouchInTextContainer, in: textContainer, fractionOfDistanceBetweenInsertionPoints: nil)
        return NSLocationInRange(indexOfCharacter, targetRange)
    }
}

extension NSMutableAttributedString {

    convenience init?(withLocalizedHTMLString: String) {

        guard let stringData = withLocalizedHTMLString.data(using: String.Encoding.utf8) else {
            return nil
        }

        let options: [NSMutableAttributedString.DocumentReadingOptionKey: Any] = [
            NSMutableAttributedString.DocumentReadingOptionKey.documentType: NSMutableAttributedString.DocumentType.html as Any,
            NSMutableAttributedString.DocumentReadingOptionKey.characterEncoding: String.Encoding.utf8.rawValue
        ]

        try? self.init(data: stringData, options: options, documentAttributes: nil)
    }
}
