//
//  UIView.swift
//  TraccarClient
//
//  Created by George Makhoul on 25/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit

extension UIView: Nameable {}

extension UIView {
    
    func show(_ show: Bool = true) {
        isHidden = !show
    }
    
    func hide() {
        isHidden = true
    }
    
    // MARK: - Drop Shadow
    func addShadow(corner: CGFloat = 0, color: UIColor, radius: CGFloat, opacity: Float, offset: CGSize) {
        addCorners(corner)
        layer.masksToBounds = false
        layer.shadowColor = color.cgColor
        layer.shadowOpacity = opacity
        layer.shadowOffset = offset
        layer.shadowRadius = radius
    }
    
    // MARK: - Round Corners
    func circle() {
        layer.cornerRadius = frame.width / 2
        clipsToBounds = true
    }
    
    func addCorners(_ radius: CGFloat) {
        layer.cornerRadius = radius
        clipsToBounds = false
    }
    
    func addCorners(corners: CACornerMask, radius: CGFloat) {
        clipsToBounds = true
        layer.cornerRadius = radius
        layer.maskedCorners = corners
    }
    
    // MARK: - Border
    func addBorder(radius: CGFloat, width: CGFloat, color: UIColor) {
        layer.cornerRadius = radius
        layer.borderWidth = width
        layer.borderColor = color.cgColor
        clipsToBounds = true
    }
    
    // MARK: - Rotate
    func rotate(from value: CGFloat, toValue: CGFloat, duration: CFTimeInterval = 0.2) {
        let animation = CABasicAnimation(keyPath: "transform.rotation")
        
        animation.fromValue = value
        animation.toValue = toValue
        animation.duration = duration
        animation.isRemovedOnCompletion = false
        animation.fillMode = CAMediaTimingFillMode.forwards
        
        layer.add(animation, forKey: nil)
    }
    
    // MARK: - Animation
    func show(with option: AnimationOptions, duration: TimeInterval, _ completion: @escaping () -> ()) {
        UIView.transition(with: self, duration: duration,
                   options: option,
                   animations: {
                    self.show()
                    completion()
                   })
    }
    
    func hide(with option: AnimationOptions, duration: TimeInterval, _ completion: @escaping () -> ()) {
        UIView.transition(with: self, duration: duration,
                   options: option,
                   animations: {
                    self.hide()
                    completion()
                   })
    }
    
    //MARK: - Add Gradient Color
    
    func addBorderGradient(startColor:UIColor, endColor: UIColor, lineWidth: CGFloat, startPoint: CGPoint, endPoint: CGPoint) {
        //This will make view border circular
        layer.cornerRadius = bounds.size.height / 2.0
        //This will hide the part outside of border, so that it would look like circle
        clipsToBounds = true
        //Create object of CAGradientLayer
        let gradient = CAGradientLayer()
        //Assign origin and size of gradient so that it will fit exactly over circular view
        gradient.frame = bounds
        //Pass the gredient colors list to gradient object
        gradient.colors = [startColor.cgColor, endColor.cgColor]
        //Point from where gradient should start
        gradient.startPoint = startPoint
        //Point where gradient should end
        gradient.endPoint = endPoint
        //Now we have to create a circular shape so that it can be added to view’s layer
        let shape = CAShapeLayer()
        //Width of circular line
        shape.lineWidth = lineWidth
        //Create circle with center same as of center of view, with radius equal to half height of view, startAngle is the angle from where circle should start, endAngle is the angle where circular path should end
        shape.path = UIBezierPath(
            arcCenter: CGPoint(x: bounds.height/2,
                               y: bounds.height/2),
            radius: bounds.height/2,
            startAngle: CGFloat(0),
            endAngle:CGFloat(CGFloat.pi * 2),
            clockwise: true).cgPath
        //the color to fill the path’s stroked outline
        shape.strokeColor = UIColor.black.cgColor
        //The color to fill the path
        shape.fillColor = UIColor.clear.cgColor
        //Apply shape to gradient layer, this will create gradient with circular border
        gradient.mask = shape
        //Finally add the gradient layer to out View
        layer.addSublayer(gradient)
    }
    
    // MARK: - Draw Dotted Line
    
    func drawDottedLine(color: UIColor, dash: NSNumber, gap: NSNumber) {
        let shapeLayer = CAShapeLayer()
        shapeLayer.strokeColor = color.cgColor
        shapeLayer.lineWidth = 1
        shapeLayer.lineDashPattern = [dash, gap]

        let path = CGMutablePath()
        path.addLines(between: [CGPoint(x: bounds.minX, y: bounds.minY), CGPoint(x: bounds.maxX, y: bounds.minY)])
        shapeLayer.path = path
        layer.addSublayer(shapeLayer)
    }
    
    // MARK: - Show/Hide No Data Found Label
    
    func showNoDataFound(_ text: String) {
        var noDataLabel = UILabel()
        noDataLabel = UILabel(frame: CGRect(x: 0, y: center.y - 100, width: UIScreen.main.bounds.width, height: 150))
//        noDataLabel.set(localized: text, color: .black, font: .semibold(18))
//        noDataLabel.align(to: .center)
        noDataLabel.numberOfLines = 0
        noDataLabel.tag = 999
        
        if viewWithTag(999) == nil {
            addSubview(noDataLabel)
        }
    }
    
    func hideNoDataFound() {
        if let viewWithTag = viewWithTag(999) {
            viewWithTag.removeFromSuperview()
        }
    }
    
    //MARK: - Load Nib
    
    class func fromNib<T: UIView>() -> T {
        return Bundle(for: T.self).loadNibNamed(String(describing: T.self), owner: nil, options: nil)![0] as! T
    }
}

